@@ -119,7 +119,8 @@ static CamelProvider exchange_provider = {
 
 	CAMEL_URL_NEED_USER | CAMEL_URL_HIDDEN_AUTH | CAMEL_URL_HIDDEN_HOST,
 
-	exchange_conf_entries
+	exchange_conf_entries,
+	NULL,
 
 	/* ... */
 };
@@ -203,7 +204,7 @@ exchange_username (const gchar *user)
 static guint
 exchange_url_hash (gconstpointer key)
 {
-	const CamelURL *u = (CamelURL *)key;
+	const CamelURL *u = (CamelURL *) key;
 	guint hash = 0;
 
 	if (u->user)
