@@ -52,18 +52,16 @@ extern CamelServiceAuthType camel_exchange_ntlm_authtype;
 
 G_DEFINE_TYPE (CamelExchangeStore, camel_exchange_store, CAMEL_TYPE_OFFLINE_STORE)
 
-/* Note: steals @name and @uri */
+/* Note: steals @display_name. */
 static CamelFolderInfo *
 make_folder_info (CamelExchangeStore *exch,
-                  gchar *name,
+                  gchar *display_name,
                   const gchar *uri,
                   gint unread_count,
                   gint flags)
 {
 	CamelFolderInfo *info;
 	const gchar *path;
-	gchar **components;
-	gchar *new_uri;
 	gchar *temp;
 
 	d(printf ("make folder info : %s flags : %d\n", name, flags));
@@ -74,17 +72,8 @@ make_folder_info (CamelExchangeStore *exch,
 	if (!path)
 		return NULL;
 
-	components = g_strsplit (uri, "/;", 2);
-	if (components[0] && components[1])
-		new_uri = g_strdup_printf ("%s/%s", components[0], components[1]);
-	else
-		new_uri = g_strdup (uri);
-	g_strfreev (components);
-
-	d(printf ("new_uri is : %s\n", new_uri));
 	info = camel_folder_info_new ();
-	info->name = name;
-	info->uri = new_uri;
+	info->display_name = display_name;
 
 	/* Process the full-path and decode if required */
 	temp = strrchr (path+2, '/');
@@ -247,22 +236,22 @@ exchange_store_finalize (GObject *object)
 	G_OBJECT_CLASS (camel_exchange_store_parent_class)->finalize (object);
 }
 
-static gboolean
-exchange_store_construct (CamelService *service,
-                          CamelSession *session,
-                          CamelProvider *provider,
-                          CamelURL *url,
-                          GError **error)
+static void
+exchange_store_constructed (GObject *object)
 {
-	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
-	CamelServiceClass *service_class;
+	CamelExchangeStore *exch;
+	CamelService *service;
+	CamelURL *url;
 	gchar *p;
 
-	service_class = CAMEL_SERVICE_CLASS (
-		camel_exchange_store_parent_class);
+	exch = CAMEL_EXCHANGE_STORE (object);
 
-	if (!service_class->construct (service, session, provider, url, error))
-		return FALSE;
+	/* Chain up to parent's constructed() method. */
+	G_OBJECT_CLASS (camel_exchange_store_parent_class)->
+		constructed (object);
+
+	service = CAMEL_SERVICE (object);
+	url = camel_service_get_camel_url (service);
 
 	exch->base_url = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
 	/* Strip path */
@@ -272,25 +261,24 @@ exchange_store_construct (CamelService *service,
 		if (p)
 			*p = '\0';
 	}
-
-	exch->storage_path = camel_session_get_storage_path (
-		session, service, error);
-
-	return (exch->storage_path != NULL);
 }
 
 static gchar *
 exchange_store_get_name (CamelService *service,
                          gboolean brief)
 {
+	CamelURL *url;
+
+	url = camel_service_get_camel_url (service);
+
 	if (brief) {
 		return g_strdup_printf (
 			_("Exchange server %s"),
-			service->url->host);
+			url->host);
 	} else {
 		return g_strdup_printf (
 			_("Exchange account for %s on %s"),
-			service->url->user, service->url->host);
+			url->user, url->host);
 	}
 }
 
@@ -303,7 +291,8 @@ exchange_store_connect_sync (CamelService *service,
 	gchar *password = NULL;
 	guint32 connect_status;
 	gboolean online_mode = FALSE;
-	CamelSession *session = camel_service_get_session (service);
+	CamelSession *session;
+	CamelURL *url;
 	GError *local_error = NULL;
 
 	/* This lock is only needed for offline operation.
@@ -311,10 +300,13 @@ exchange_store_connect_sync (CamelService *service,
 
 	g_mutex_lock (exch->connect_lock);
 
+	url = camel_service_get_camel_url (service);
+	session = camel_service_get_session (service);
+
 	online_mode = camel_session_get_online (session);
 
 	if (online_mode) {
-		if (!service->url->passwd) {
+		if (!url->passwd) {
 			gchar *prompt;
 			guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
 
@@ -322,23 +314,23 @@ exchange_store_connect_sync (CamelService *service,
 				prompt_flags |= CAMEL_SESSION_PASSWORD_REPROMPT;
 
 			prompt = camel_session_build_password_prompt (
-				"Exchange", service->url->user, service->url->host);
+				"Exchange", url->user, url->host);
 
-			service->url->passwd = camel_session_get_password (
+			url->passwd = camel_session_get_password (
 				session, service, "Exchange", prompt,
 				"password", prompt_flags, error);
 
 			g_free (prompt);
 
-			exch->reprompt_password = service->url->passwd == NULL;
+			exch->reprompt_password = url->passwd == NULL;
 		}
 
-		if (service->url->passwd == NULL) {
+		if (url->passwd == NULL) {
 			g_mutex_unlock (exch->connect_lock);
 			return FALSE;
 		}
 
-		password = service->url->passwd;
+		password = url->passwd;
 	}
 
 	/* Initialize the stub connection */
@@ -360,9 +352,9 @@ exchange_store_connect_sync (CamelService *service,
 	if (!connect_status) {
 		exch->reprompt_password = TRUE;
 
-		if (service->url->passwd) {
-			g_free (service->url->passwd);
-			service->url->passwd = NULL;
+		if (url->passwd) {
+			g_free (url->passwd);
+			url->passwd = NULL;
 		}
 
 		g_clear_error (error);
@@ -409,12 +401,15 @@ exchange_store_can_refresh_folder (CamelStore *store,
                                    GError **error)
 {
 	CamelStoreClass *store_class;
+	CamelURL *url;
 	gboolean res;
 
 	store_class = CAMEL_STORE_CLASS (camel_exchange_store_parent_class);
 
+	url = camel_service_get_camel_url (CAMEL_SERVICE (store));
+
 	res = store_class->can_refresh_folder (store, info, error) ||
-	      (camel_url_get_param (((CamelService *)store)->url, "check_all") != NULL);
+	      (camel_url_get_param (url, "check_all") != NULL);
 
 	return res;
 }
@@ -444,17 +439,22 @@ exchange_store_get_folder_sync (CamelStore *store,
                                 GError **error)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
+	CamelService *service;
 	CamelFolder *folder;
+	const gchar *user_data_dir;
 	const gchar *short_name;
 	gchar *folder_dir;
 
 	RETURN_VAL_IF_NOT_CONNECTED (exch, cancellable, error, NULL);
 
+	service = CAMEL_SERVICE (store);
+	user_data_dir = camel_service_get_user_data_dir (service);
+
 	if (!folder_name || !*folder_name || g_ascii_strcasecmp (folder_name, "inbox") == 0)
 		folder_name = "personal/Inbox";
 
 	folder_dir = exchange_store_path_to_physical (
-		exch->storage_path, folder_name);
+		user_data_dir, folder_name);
 
 	if (!camel_exchange_store_connected (exch, cancellable, NULL)) {
 		if (!folder_dir || !g_file_test (folder_dir, G_FILE_TEST_IS_DIR)) {
@@ -485,7 +485,7 @@ exchange_store_get_folder_sync (CamelStore *store,
 
 	folder = g_object_new (
 		CAMEL_TYPE_EXCHANGE_FOLDER,
-		"name", short_name, "full-name", folder_name,
+		"display-name", short_name, "full-name", folder_name,
 		"parent-store", store, NULL);
 	g_hash_table_insert (exch->folders, g_strdup (folder_name), folder);
 	g_mutex_unlock (exch->folders_lock);
@@ -777,9 +777,9 @@ camel_exchange_store_class_init (CamelExchangeStoreClass *class)
 
 	object_class = G_OBJECT_CLASS (class);
 	object_class->finalize = exchange_store_finalize;
+	object_class->constructed = exchange_store_constructed;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
-	service_class->construct = exchange_store_construct;
 	service_class->get_name = exchange_store_get_name;
 	service_class->connect_sync = exchange_store_connect_sync;
 	service_class->disconnect_sync = exchange_store_disconnect_sync;
@@ -823,15 +823,17 @@ camel_exchange_store_connected (CamelExchangeStore *store,
                                 GCancellable *cancellable,
                                 GError **error)
 {
+	CamelServiceConnectionStatus status;
 	CamelService *service;
 	CamelSession *session;
 
 	g_return_val_if_fail (CAMEL_IS_EXCHANGE_STORE (store), FALSE);
 
 	service = CAMEL_SERVICE (store);
-	session = service->session;
+	session = camel_service_get_session (service);
+	status = camel_service_get_connection_status (service);
 
-	if (service->status != CAMEL_SERVICE_CONNECTED &&
+	if (status != CAMEL_SERVICE_CONNECTED &&
 	    camel_session_get_online (session) &&
 	    !camel_service_connect_sync (service, error)) {
 		return FALSE;
